import {
	AutocompleteInteraction,
	ChatInputCommandInteraction,
	EmbedBuilder,
	SlashCommandBuilder,
	TextChannel,
} from "npm:discord.js";

import { Storage } from "../storage.ts";
import { _1d, _1min, Channel, collect, days_to_ms, log_from, trimmed } from "../utils.ts";
import { voids } from "../utils.ts";

const log = log_from(import.meta);

// exported

export function declare_commands(subjects: Set<string>) {
	return [devoir_command(subjects), adm_command()];
}

export async function handle_command(
	interaction: ChatInputCommandInteraction,
	storage: Storage,
	update_display: Channel,
) {
	log("Received command", interaction.commandName);
	if (interaction.commandName === "devoir") return await handle_command_devoir(interaction, storage, update_display);
	if (interaction.commandName === "adm") return await handle_command_adm(interaction, storage);
	log("Unknown command", interaction.commandName);
}

export async function handle_autocomplete(interaction: AutocompleteInteraction, storage: Storage) {
	log("Auto completing.");
	if (interaction.commandName === "devoir") return await handle_autocomplete_devoir(interaction, storage);
	log("Unknown command", interaction.commandName);
}

// declare

function adm_command() {
	return new SlashCommandBuilder().setName("adm")
		.setDescription("Commandes d'administration.")
		.addSubcommand((command) =>
			command.setName("ajouter-feed")
				.setDescription("Ajoute un salon comme feed de notifications.")
				.addChannelOption((option) =>
					option.setName("salon")
						.setDescription("Salon du nouveau feed.")
						.setRequired(true)
				)
		);
}

function devoir_command(subjects: Set<string>) {
	const subjects_as_choices = [...subjects.values()].map((name) => ({ name, value: name }));
	return new SlashCommandBuilder()
		.setName("devoir")
		.setDescription("Manipuler les devoirs à effectuer.")
		.addSubcommand((command) =>
			command.setName("ajouter")
				.setDescription("Ajouter un nouveau devoir à effectuer.")
				.addStringOption((option) =>
					option.setName("matière")
						.setDescription("Matière du devoir à effectuer.")
						.addChoices(...subjects_as_choices)
						.setRequired(true)
				)
				.addNumberOption((option) =>
					option.setName("jours")
						.setMinValue(0)
						.setDescription("Nombre de jours avant la date du rendu.")
						.setRequired(true)
				)
				.addStringOption((option) =>
					option.setName("description")
						.setDescription("Description du devoir à effectuer.")
						.setRequired(true)
				)
		).addSubcommand((command) =>
			command.setName("retirer")
				.setDescription("Retirer un devoir.")
				.addStringOption((option) =>
					option.setName("devoir")
						.setDescription("Devoir à retirer.")
						.setAutocomplete(true)
						.setRequired(true)
				)
		).addSubcommand((command) =>
			command.setName("éditer")
				.setDescription("Éditer un devoir.")
				.addStringOption((option) =>
					option.setName("devoir")
						.setDescription("Devoir à éditer.")
						.setAutocomplete(true)
						.setRequired(true)
				)
				.addStringOption((option) =>
					option.setName("matière")
						.setDescription("Nouvelle matière.")
						.addChoices(...subjects_as_choices)
						.setRequired(false)
				)
				.addNumberOption((option) =>
					option.setName("jours")
						.setMinValue(0)
						.setDescription("Nouveau nombre de jours avant la date du rendu.")
						.setRequired(false)
				)
				.addStringOption((option) =>
					option.setName("description")
						.setDescription("Nouvelle description.")
						.setRequired(false)
				)
		);
}

// handling

async function handle_command_devoir(
	interaction: ChatInputCommandInteraction,
	storage: Storage,
	update_display: Channel,
) {
	const subcommand = interaction.options.getSubcommand(true);
	if (subcommand === "ajouter") return await handle_command_devoir_ajouter(interaction, storage, update_display);
	if (subcommand === "retirer") return await handle_command_devoir_retirer(interaction, storage, update_display);
	if (subcommand === "éditer") return await handle_command_devoir_editer(interaction, storage, update_display);
	return log("Unknown devoir sub command", subcommand);
}

async function handle_command_devoir_editer(
	interaction: ChatInputCommandInteraction,
	storage: Storage,
	update_display: Channel,
) {
	const id = interaction.options.getString("devoir", true);
	const subject = interaction.options.getString("matière", false);
	if (subject !== null) storage.devoirs.update({ id }, (d) => d.subject = subject);
	const description = interaction.options.getString("description", false);
	if (description !== null) storage.devoirs.update({ id }, (d) => d.description = description);
	const days = interaction.options.getNumber("jours", false);
	if (days !== null) storage.devoirs.update({ id }, (d) => d.date = Date.now() + days_to_ms(days));
	const embed = new EmbedBuilder()
		.setTitle("`🔧` Édité")
		.setDescription(`Devoir mis à jour avec succès.`)
		.setFooter({ text: `(devoir id:${id})` });
	await interaction.reply({ embeds: [embed], ephemeral: true });
	update_display.send();
}

async function handle_command_devoir_retirer(
	interaction: ChatInputCommandInteraction,
	storage: Storage,
	update_display: Channel,
) {
	const id = interaction.options.getString("devoir", true);
	await storage.devoirs.delete({ id });
	const embed = new EmbedBuilder()
		.setTitle("`🗑️` Retiré")
		.setDescription(`Devoir supprimé avec succès.`)
		.setFooter({ text: `(devoir id:${id})` });
	await interaction.reply({ embeds: [embed], ephemeral: true });
	update_display.send();
}

async function handle_command_devoir_ajouter(
	interaction: ChatInputCommandInteraction,
	storage: Storage,
	update_display: Channel,
) {
	const subject = interaction.options.getString("matière", true);
	const days = interaction.options.getNumber("jours", true);
	const description = interaction.options.getString("description", true);
	const date = Date.now() + days_to_ms(days);
	const { id } = await storage.devoirs.add({ date, description, subject });
	const embed = new EmbedBuilder()
		.setTitle("`🌟` Ajouté")
		.setDescription(`Nouveau devoir ajouté avec succès.`)
		.setFooter({ text: `(devoir id:${id})` });
	await interaction.reply({ embeds: [embed], ephemeral: true });
	update_display.send();
}

async function handle_command_adm(interaction: ChatInputCommandInteraction, storage: Storage) {
	const subcommand = interaction.options.getSubcommand(true);
	if (subcommand === "ajouter-feed") return await handle_command_adm_ajouter_feed(interaction, storage);
	return log("Unknown adm sub command", subcommand);
}

async function handle_command_adm_ajouter_feed(interaction: ChatInputCommandInteraction, storage: Storage) {
	const channel = interaction.options.getChannel("salon", true);
	const is_text_channel = channel instanceof TextChannel;
	if (!is_text_channel) return voids(await interaction.reply("Channel must be text."));
	const board_message = await channel.send("[board]");
	const feed_id = await storage.feeds.add({
		channel_id: channel.id,
		board_message_id: board_message.id,
		notification_ids: new Set(),
	});
	await interaction.reply("Added feed " + feed_id.id);
}

// autocompletion

async function handle_autocomplete_devoir(interaction: AutocompleteInteraction, storage: Storage) {
	const subcommand = interaction.options.getSubcommand(true);
	if (subcommand === "retirer") return await autocomplete_with_devoirs(storage, interaction);
	if (subcommand === "éditer") return await autocomplete_with_devoirs(storage, interaction);
	log("Unknown devoir sub command", subcommand);
}

async function autocomplete_with_devoirs(storage: Storage, interaction: AutocompleteInteraction) {
	const devoirs = await collect(storage.devoirs.list());
	const mapped = devoirs.map(([{ id }, { subject, description }]) => ({
		name: trimmed(`[${subject}] ${description}`, 100),
		value: id,
	}));
	await interaction.respond(mapped);
}
